/**
 * Kosaraju
 *
 * Complexity (Time): O(n + m)
 * Complexity (Space): O(n + m)
 */

stack<int> S;
vector<int> graph[MAX];
vector<int> transp[MAX];

bool cont[MAX];

// Traverse a SCC
void dfs(int x) {
  // x belong to current scc
  cont[x] = true;

  for (auto i : transp[x])
    if (!cont[i])
      dfs(i);
}


// Fill stack with DFS starting points to find SCC
void fill_stack(int x) {
  cont[x] = true;

  for (auto i : graph[x])
    if (!cont[i])
      fill_stack(i);

  S.push(x);
}


// Return number of SCC of a graph with n vertices
int kosaraju(int n) {
  int scc = 0;

  memset(cont, 0, sizeof cont);
  for (int i = 0; i < n; ++i)
    if (!cont[i])
      fill_stack(i);

  // Transpose graph
  for (int i = 0; i < n; ++i)
    for (auto j : graph[i])
      transp[j].push_back(i);

  // Count SCC
  memset(cont, 0, sizeof cont);
  while (!S.empty()) {
    int v = S.top();
    S.pop();

    if (!cont[v]) {
      dfs(v);
      scc++;
    }
  }

  return scc;
}
